(   Title:  IEEE 754-2008 tests
     File:  ieeefp-test.fs
   Author:  David N. Williams
  Version:  0.8.0
  License:  Public Domain
  Revised:  December 23, 2020

The revision date may reflect cosmetic changes not logged below.

Version 0.8.0
09Sep20 * Started consolidation of fatan2-test.fs 0.1.0,
          ieee-fprox-test.fs 0.5.2, and ieee-arith-test.fs 0.5.2.
10Sep20 * Finished consolidation.
27Sep20 * REDEFINED-WARNING-OFF instead of REDEFINED-WARNING off.
22Dec20 * Added constant definitions with 50 decimal digits.
)

\ For debugging:
[UNDEFINED] \\ [IF]
: \\  ( -- )  -1 parse 2drop BEGIN refill 0= UNTIL ; [THEN]

\ Causes pfe to load floating point:
\ s" FLOATING-EXT" environment? [IF] drop [THEN]

35 SET-PRECISION

\ TRUE CONSTANT USE-TTESTER
FALSE CONSTANT USE-TTESTER

USE-TTESTER [IF]
INCLUDE ttester.fs
INCLUDE tester-display.fs
[ELSE]
INCLUDE ttester-xf.fs
INCLUDE tester-display.fs
INCLUDE xtester-errors.fs
INCLUDE ftester-errors.fs
[THEN]
true verbose !
REDEFINED-WARNING-OFF
decimal

USE-TTESTER [IF]

\ The ttester default for testing floating-point equality is SET-EXACT. 
\ If you do not want that, uncomment the following line:
\ SET-NEAR

variable #errors    0 #errors !
:noname  ( c-addr u -- )
(
Display an error message followed by the line that had the
error.
)
  1 #errors +! error1 ; error-xt !

: ?.errors  ( -- )  VERBOSE @
  IF blue-text ." #ERRORS: " normal-text #errors @ . THEN ;

[ELSE]  \ USE TTESTER-XF

\ The ttester-xf default for floating-point equality is SET-FT-MODE-EXACT.
\ If you don't want that, uncomment one of the following lines:
\ SET-FT-MODE-ABS
\ SET-FT-MODE-REL
\ SET-FT-MODE-AREL
\ SET-FT-MODE-REL0

: ?.errors  ( -- )  ?.xt-errors s"     " ?type ?.ft-errors ?.cr ;

[THEN]

:noname  ( -- fp.separate? )
  depth >r 1e depth >r fdrop 2r> = ; execute
CONSTANT FSTACK-SEPARATE?

?.first-cr
?." Floating-point and data stacks are " FSTACK-SEPARATE?
[IF]   ?." *separate*.
[ELSE] ?." *not separate*. [THEN]


\ *** fconstants ***
(
This section will eventually be replaced by reliable, tested definitions
of floating-point constants.
)

\ FABS should be superfluous here:

0e fabs    FCONSTANT +0 
+0 fnegate FCONSTANT -0

[UNDEFINED] +inf [IF]  1e 0e f/ FCONSTANT +inf [THEN]
[UNDEFINED] -inf [IF] -1e 0e f/ FCONSTANT -inf [THEN]

\ FABS is not superflous here, because the sign of 0/0 is not specified
\ by IEEE, and is actually different in Mac OS X ppc/intel (+/-), both
\ for gforth and pfe:

[UNDEFINED] +nan [IF] 0e 0e f/ fabs          FCONSTANT +nan [THEN]
[UNDEFINED] -nan [IF] 0e 0e f/ fabs fnegate  FCONSTANT -nan [THEN]

true [IF]

 0.31415926535897932384626433832795028841971693993751e1 FCONSTANT  pi
-0.31415926535897932384626433832795028841971693993751e1 FCONSTANT -pi

\ Defining fractions of pi in terms of pi avoids most of the decimal
\ to binary conversions.  Suggested by Bernd Paysan.  The fact that both
\ branches of this conditional work validates his suggestion.

 pi 4e f/   FCONSTANT   pi/4
-pi 4e f/   FCONSTANT  -pi/4
 pi 2e f/   FCONSTANT   pi/2
-pi 2e f/   FCONSTANT  -pi/2
 pi/2 3e f* FCONSTANT  3pi/2
 pi/4 3e f* FCONSTANT  3pi/4
-pi/4 3e f* FCONSTANT -3pi/4

[ELSE]

\ These 50-digit constants are good for binary 128, which is about 34
\ decimal digits.

 0.31415926535897932384626433832795028841971693993751e1 FCONSTANT  pi
-0.31415926535897932384626433832795028841971693993751e1 FCONSTANT -pi
 0.78539816339744830961566084581987572104929234984378e0 FCONSTANT  pi/4
-0.78539816339744830961566084581987572104929234984378e0 FCONSTANT -pi/4
 0.15707963267948966192313216916397514420985846996876e1 FCONSTANT  pi/2
-0.15707963267948966192313216916397514420985846996876e1 FCONSTANT -pi/2
 0.47123889803846898576939650749192543262957540990627e1 FCONSTANT  3pi/2
-0.47123889803846898576939650749192543262957540990627e1 FCONSTANT -3pi/2
 0.23561944901923449288469825374596271631478770495313e1 FCONSTANT  3pi/4
-0.23561944901923449288469825374596271631478770495313e1 FCONSTANT -3pi/4

[THEN]

?." *** FATAN2 TESTS ***
(
Tests the ISO C / Single UNIX 3 specification for the ANS Forth
Floating-Ext word:
)
\ FATAN2 ( f: y x -- radians ) or ( y x -- radians )
(
It expects the return value to be the principal angle, and includes a
number of tests with +/-0, +/-nan, and +/-inf that are not required by
the ANS Forth 94 standard.
)

TESTING normal values

\ Input x-y values taken from Ed, comp.lang.forth,
\ "Re: Ambiguity in FATAN2", 14Mar09, 17Mar09:
\   y   x              rad          deg
t{  0e  1e fatan2 ->   0e   }t    \   0
t{  1e  1e fatan2 ->   pi/4 }t    \  45
t{  1e  0e fatan2 ->   pi/2 }t    \  90
t{ -1e -1e fatan2 -> -3pi/4 }t    \ 135
t{  0e -1e fatan2 ->   pi   }t    \ 180
t{ -1e  1e fatan2 ->  -pi/4 }t    \ 225
t{ -1e  0e fatan2 ->  -pi/2 }t    \ 270
t{ -1e  1e fatan2 ->  -pi/4 }t    \ 315

TESTING Single UNIX 3 special values spec

\ ISO C / Single UNIX Specification Version 3:
\   http://www.unix.org/single_unix_specification/
\ Select "Topic", then "Math Interfaces", then "atan2()":
\   http://www.opengroup.org/onlinepubs/009695399/functions/atan2f.html

\ If y is +/-0 and x is < 0, +/-pi shall be returned.
t{  0e -1e fatan2 ->  pi }t
t{ -0e -1e fatan2 -> -pi }t

\ If y is +/-0 and x is > 0, +/-0 shall be returned.
t{  0e  1e fatan2 ->  0e }t
t{ -0e  1e fatan2 -> -0e }t

\ If y is < 0 and x is +/-0, -pi/2 shall be returned.
t{ -1e  0e fatan2 -> -pi/2 }t
t{ -1e -0e fatan2 -> -pi/2 }t

\ If y is > 0 and x is +/-0, pi/2 shall be returned.
t{  1e  0e fatan2 -> pi/2 }t
t{  1e -0e fatan2 -> pi/2 }t

TESTING Single UNIX 3 special values optional spec

\ Optional ISO C / single UNIX specs:

\ If either x or y is nan, a nan shall be returned.
t{ +nan  1e fatan2 -> +nan }t
t{  1e +nan fatan2 -> +nan }t
t{ +nan +nan fatan2 -> +nan }t

\ If y is +/-0 and x is -0, +/-pi shall be returned.
t{  0e -0e fatan2 ->  pi }t
t{ -0e -0e fatan2 -> -pi }t

\ If y is +/-0 and x is +0, +/-0 shall be returned.
t{  0e  0e fatan2 -> +0e }t
t{ -0e  0e fatan2 -> -0e }t

\ For finite values of +/-y > 0, if x is -inf, +/-pi shall be returned.
t{  1e -inf fatan2 ->  pi }t
t{ -1e -inf fatan2 -> -pi }t

\ For finite values of +/-y > 0, if x is +inf, +/-0 shall be returned.
t{  1e +inf fatan2 -> +0e }t
t{ -1e +inf fatan2 -> -0e }t

\ For finite values of x, if y is +/-inf, +/-pi/2 shall be returned.
t{ +inf  1e fatan2 ->  pi/2 }t
t{ +inf -1e fatan2 ->  pi/2 }t
t{ +inf  0e fatan2 ->  pi/2 }t
t{ +inf -0e fatan2 ->  pi/2 }t
t{ -inf  1e fatan2 -> -pi/2 }t
t{ -inf -1e fatan2 -> -pi/2 }t
t{ -inf  0e fatan2 -> -pi/2 }t
t{ -inf -0e fatan2 -> -pi/2 }t

\ If y is +/-inf and x is -inf, +/-3pi/4 shall be returned.
t{ +inf -inf fatan2 ->  3pi/4 }t
t{ -inf -inf fatan2 -> -3pi/4 }t

\ If y is +/-inf and x is +inf, +/-pi/4 shall be returned.
t{ +inf +inf fatan2 ->  pi/4 }t
t{ -inf +inf fatan2 -> -pi/4 }t


?." *** IEEE-FPROX TESTS ***

TESTING F~ exact tolerance

t{ +0 +0  +0 f~ -> true }t
t{ +0 -0  +0 f~ -> false }t
t{ -0 +0  +0 f~ -> false }t
t{ -0 -0  +0 f~ -> true }t

t{  7e -2e  +0 f~ -> false }t
t{ -2e  7e  +0 f~ -> false }t
t{  7e  7e  +0 f~ -> true }t
   
t{  7e +inf  +0 f~ -> false }t
t{ +inf 7e   +0 f~ -> false }t
t{  7e -inf  +0 f~ -> false }t
t{ -inf 7e   +0 f~ -> false }t

t{ +inf +inf  +0 f~ -> true }t
t{ +inf -inf  +0 f~ -> false }t
t{ -inf +inf  +0 f~ -> false }t
t{ -inf -inf  +0 f~ -> true }t

t{ +nan 7e   +0 f~ -> false }t
t{ -nan 7e   +0 f~ -> false }t
t{  7e +nan  +0 f~ -> false }t
t{  7e -nan  +0 f~ -> false }t

t{ +nan +nan  +0 f~ -> true }t
t{ -nan +nan  +0 f~ -> false }t
t{ +nan -nan  +0 f~ -> false }t
t{ -nan -nan  +0 f~ -> true }t

t{ +inf +nan  +0 f~ -> false }t
t{ -inf +nan  +0 f~ -> false }t
t{ +inf -nan  +0 f~ -> false }t
t{ -inf -nan  +0 f~ -> false }t

t{ +nan +inf  +0 f~ -> false }t
t{ -nan +inf  +0 f~ -> false }t
t{ +nan -inf  +0 f~ -> false }t
t{ -nan -inf  +0 f~ -> false }t

t{ +0 +0  -0 f~ -> true }t
t{ +0 -0  -0 f~ -> false }t
t{ -0 +0  -0 f~ -> false }t
t{ -0 -0  -0 f~ -> true }t

t{  7e -2e  -0 f~ -> false }t
t{ -2e  7e  -0 f~ -> false }t
t{  7e  7e  -0 f~ -> true }t

t{  7e +inf  -0 f~ -> false }t
t{ +inf 7e   -0 f~ -> false }t
t{  7e -inf  -0 f~ -> false }t
t{ -inf 7e   -0 f~ -> false }t

t{ +inf -inf  -0 f~ -> false }t
t{ -inf +inf  -0 f~ -> false }t
t{ -inf -inf  -0 f~ -> true }t
t{ +inf +inf  -0 f~ -> true }t

t{ +nan 7e   -0 f~ -> false }t
t{ -nan 7e   -0 f~ -> false }t
t{  7e +nan  -0 f~ -> false }t
t{  7e -nan  -0 f~ -> false }t

t{ +nan +nan  -0 f~ -> true }t
t{ -nan +nan  -0 f~ -> false }t
t{ +nan -nan  -0 f~ -> false }t
t{ -nan -nan  -0 f~ -> true }t

t{ +inf +nan  -0 f~ -> false }t
t{ -inf +nan  -0 f~ -> false }t
t{ +inf -nan  -0 f~ -> false }t
t{ -inf -nan  -0 f~ -> false }t

t{ +nan +inf  -0 f~ -> false }t
t{ -nan +inf  -0 f~ -> false }t
t{ +nan -inf  -0 f~ -> false }t
t{ -nan -inf  -0 f~ -> false }t

TESTING F~ absolute tolerance

t{ +0 +0  7e f~ -> true }t
t{ +0 -0  7e f~ -> true }t
t{ -0 +0  7e f~ -> true }t
t{ -0 -0  7e f~ -> true }t

t{  7e +inf  7e f~ -> false }t
t{ +inf 7e   7e f~ -> false }t
t{  7e -inf  7e f~ -> false }t
t{ -inf 7e   7e f~ -> false }t

t{ +inf +inf  7e f~ -> false }t
t{ +inf -inf  7e f~ -> false }t
t{ -inf +inf  7e f~ -> false }t
t{ -inf -inf  7e f~ -> false }t

t{ +nan 7e   7e f~ -> false }t
t{ -nan 7e   7e f~ -> false }t
t{  7e +nan  7e f~ -> false }t
t{  7e -nan  7e f~ -> false }t

t{ +nan +nan  7e f~ -> false }t
t{ -nan +nan  7e f~ -> false }t
t{ +nan -nan  7e f~ -> false }t
t{ -nan -nan  7e f~ -> false }t

t{ +inf +nan  7e f~ -> false }t
t{ -inf +nan  7e f~ -> false }t
t{ +inf -nan  7e f~ -> false }t
t{ -inf -nan  7e f~ -> false }t

t{ +nan +inf  7e f~ -> false }t
t{ -nan +inf  7e f~ -> false }t
t{ +nan -inf  7e f~ -> false }t
t{ -nan -inf  7e f~ -> false }t

t{ +0 +0  +inf f~ -> true }t
t{ +0 -0  +inf f~ -> true }t
t{ -0 +0  +inf f~ -> true }t
t{ -0 -0  +inf f~ -> true }t

t{  7e -2e  +inf f~ -> true }t
t{ -2e  7e  +inf f~ -> true }t
t{  7e  7e  +inf f~ -> true }t

t{  7e +inf  +inf f~ -> false }t
t{ +inf 7e   +inf f~ -> false }t
t{  7e -inf  +inf f~ -> false }t
t{ -inf 7e   +inf f~ -> false }t

t{ +inf +inf  +inf f~ -> false }t
t{ +inf -inf  +inf f~ -> false }t
t{ -inf +inf  +inf f~ -> false }t
t{ -inf -inf  +inf f~ -> false }t

t{ +nan 7e   +inf f~ -> false }t
t{ -nan 7e   +inf f~ -> false }t
t{  7e +nan  +inf f~ -> false }t
t{  7e -nan  +inf f~ -> false }t

t{ +nan +nan  +inf f~ -> false }t
t{ -nan +nan  +inf f~ -> false }t
t{ +nan -nan  +inf f~ -> false }t
t{ -nan -nan  +inf f~ -> false }t

t{ +inf +nan  +inf f~ -> false }t
t{ -inf +nan  +inf f~ -> false }t
t{ +inf -nan  +inf f~ -> false }t
t{ -inf -nan  +inf f~ -> false }t

t{ +nan +inf  +inf f~ -> false }t
t{ -nan +inf  +inf f~ -> false }t
t{ +nan -inf  +inf f~ -> false }t
t{ -nan -inf  +inf f~ -> false }t

t{ +0 +0  +nan f~ -> false }t
t{ +0 -0  +nan f~ -> false }t
t{ -0 +0  +nan f~ -> false }t
t{ -0 -0  +nan f~ -> false }t

t{  7e -2e  +nan f~ -> false }t
t{ -2e  7e  +nan f~ -> false }t
t{  7e  7e  +nan f~ -> false }t

t{  7e +inf  +nan f~ -> false }t
t{ +inf 7e   +nan f~ -> false }t
t{  7e -inf  +nan f~ -> false }t
t{ -inf 7e   +nan f~ -> false }t

t{ +inf +inf  +nan f~ -> false }t
t{ +inf -inf  +nan f~ -> false }t
t{ -inf +inf  +nan f~ -> false }t
t{ -inf -inf  +nan f~ -> false }t

t{ +nan 7e   +nan f~ -> false }t
t{ -nan 7e   +nan f~ -> false }t
t{  7e +nan  +nan f~ -> false }t
t{  7e -nan  +nan f~ -> false }t

t{ +nan +nan  +nan f~ -> false }t
t{ -nan +nan  +nan f~ -> false }t
t{ +nan -nan  +nan f~ -> false }t
t{ -nan -nan  +nan f~ -> false }t

t{ +inf +nan  +nan f~ -> false }t
t{ -inf +nan  +nan f~ -> false }t
t{ +inf -nan  +nan f~ -> false }t
t{ -inf -nan  +nan f~ -> false }t

t{ +nan +inf  +nan f~ -> false }t
t{ -nan +inf  +nan f~ -> false }t
t{ +nan -inf  +nan f~ -> false }t
t{ -nan -inf  +nan f~ -> false }t

TESTING F~ relative tolerance

t{ +0 +0  -7e f~ -> false }t
t{ +0 -0  -7e f~ -> false }t
t{ -0 +0  -7e f~ -> false }t
t{ -0 -0  -7e f~ -> false }t

t{  7e +inf  -7e f~ -> false }t
t{ +inf 7e   -7e f~ -> false }t
t{  7e -inf  -7e f~ -> false }t
t{ -inf 7e   -7e f~ -> false }t

t{ +inf +inf  -7e f~ -> false }t
t{ +inf -inf  -7e f~ -> false }t
t{ -inf +inf  -7e f~ -> false }t
t{ -inf -inf  -7e f~ -> false }t

t{ +nan 7e   -7e f~ -> false }t
t{ -nan 7e   -7e f~ -> false }t
t{  7e +nan  -7e f~ -> false }t
t{  7e -nan  -7e f~ -> false }t

t{ +nan +nan  -7e f~ -> false }t
t{ -nan +nan  -7e f~ -> false }t
t{ +nan -nan  -7e f~ -> false }t
t{ -nan -nan  -7e f~ -> false }t

t{ +inf +nan  -7e f~ -> false }t
t{ -inf +nan  -7e f~ -> false }t
t{ +inf -nan  -7e f~ -> false }t
t{ -inf -nan  -7e f~ -> false }t

t{ +nan +inf  -7e f~ -> false }t
t{ -nan +inf  -7e f~ -> false }t
t{ +nan -inf  -7e f~ -> false }t
t{ -nan -inf  -7e f~ -> false }t

t{ +0 +0  -inf f~ -> false }t
t{ +0 -0  -inf f~ -> false }t
t{ -0 +0  -inf f~ -> false }t
t{ -0 -0  -inf f~ -> false }t

t{  7e -2e  -inf f~ -> true }t
t{ -2e  7e  -inf f~ -> true }t
t{  7e  7e  -inf f~ -> true }t

t{  7e +inf  -inf f~ -> false }t
t{ +inf 7e   -inf f~ -> false }t
t{  7e -inf  -inf f~ -> false }t
t{ -inf 7e   -inf f~ -> false }t

t{ +inf +inf  -inf f~ -> false }t
t{ +inf -inf  -inf f~ -> false }t
t{ -inf +inf  -inf f~ -> false }t
t{ -inf -inf  -inf f~ -> false }t

t{ +nan 7e   -inf f~ -> false }t
t{ -nan 7e   -inf f~ -> false }t
t{  7e +nan  -inf f~ -> false }t
t{  7e -nan  -inf f~ -> false }t

t{ +nan +nan  -inf f~ -> false }t
t{ -nan +nan  -inf f~ -> false }t
t{ +nan -nan  -inf f~ -> false }t
t{ -nan -nan  -inf f~ -> false }t

t{ +inf +nan  -inf f~ -> false }t
t{ -inf +nan  -inf f~ -> false }t
t{ +inf -nan  -inf f~ -> false }t
t{ -inf -nan  -inf f~ -> false }t

t{ +nan +inf  -inf f~ -> false }t
t{ -nan +inf  -inf f~ -> false }t
t{ +nan -inf  -inf f~ -> false }t
t{ -nan -inf  -inf f~ -> false }t

t{ +0 +0  +nan f~ -> false }t
t{ +0 -0  +nan f~ -> false }t
t{ -0 +0  +nan f~ -> false }t
t{ -0 -0  +nan f~ -> false }t

t{  7e -2e  -nan f~ -> false }t
t{ -2e  7e  -nan f~ -> false }t
t{  7e  7e  -nan f~ -> false }t

t{  7e +inf  -nan f~ -> false }t
t{ +inf 7e   -nan f~ -> false }t
t{  7e -inf  -nan f~ -> false }t
t{ -inf 7e   -nan f~ -> false }t

t{ +inf +inf  -nan f~ -> false }t
t{ +inf -inf  -nan f~ -> false }t
t{ -inf +inf  -nan f~ -> false }t
t{ -inf -inf  -nan f~ -> false }t

t{ +nan 7e   -nan f~ -> false }t
t{ -nan 7e   -nan f~ -> false }t
t{  7e +nan  -nan f~ -> false }t
t{  7e -nan  -nan f~ -> false }t

t{ +nan +nan  -nan f~ -> false }t
t{ -nan +nan  -nan f~ -> false }t
t{ +nan -nan  -nan f~ -> false }t
t{ -nan -nan  -nan f~ -> false }t

t{ +inf +nan  -nan f~ -> false }t
t{ -inf +nan  -nan f~ -> false }t
t{ +inf -nan  -nan f~ -> false }t
t{ -inf -nan  -nan f~ -> false }t

t{ +nan +inf  -nan f~ -> false }t
t{ -nan +inf  -nan f~ -> false }t
t{ +nan -inf  -nan f~ -> false }t
t{ -nan -inf  -nan f~ -> false }t


?." *** IEEE-ARITH TESTS ***

\ The following is just for testing sqrt(-1)!  But maybe FNAN? can also
\ be used to improve some of the other tests.

[UNDEFINED] fnan? [IF]
: fdatum=  ( f: r1 r2 -- s: flag )  0e f~ ;

\ maybe this can be improved
: fnan?  ( f: r -- s: isnan? )
  fdup +0   fdatum=
  fdup -0   fdatum= or
  fdup -inf fdatum= or
  fdup +inf fdatum= or IF fdrop false EXIT THEN
  +inf f< 0= ;
[THEN]

t{ 1e +inf f< -> true }t
t{ +0   fnan? -> false }t
t{ -0   fnan? -> false }t
t{ +inf fnan? -> false }t
t{ -inf fnan? -> false }t
t{  1e  fnan? -> false }t
t{ +nan fnan? -> true }t
t{ -nan fnan? -> true }t

TESTING F+

\ IEEE-2008 6.3

t{ +0 +0 f+ -> +0 }t
t{ +0 -0 f+ -> +0 }t
t{ -0 +0 f+ -> +0 }t
t{ -0 -0 f+ -> -0 }t

t{ +nan  2e  f+ -> +nan }t
t{ -nan  2e  f+ -> -nan }t
t{  3e  +nan f+ -> +nan }t
t{  3e  -nan f+ -> -nan }t

t{ +nan +nan f+ -> +nan }t
t{ -nan +nan f+ fabs -> +nan }t
t{ +nan -nan f+ fabs -> +nan }t
t{ -nan -nan f+ -> -nan }t

t{  2e  +inf f+ -> +inf }t
t{ +inf  7e  f+ -> +inf }t
t{  2e  -inf f+ -> -inf }t
t{ -inf  7e  f+ -> -inf }t

t{ +nan +inf f+ -> +nan }t
t{ +inf +nan f+ -> +nan }t
t{ +nan -inf f+ -> +nan }t
t{ -inf +nan f+ -> +nan }t
t{ -nan +inf f+ -> -nan }t
t{ +inf -nan f+ -> -nan }t
t{ -nan -inf f+ -> -nan }t
t{ -inf -nan f+ -> -nan }t

t{ +inf +inf f+      -> +inf }t
t{ +inf -inf f+ fabs -> +nan }t
t{ -inf +inf f+ fabs -> +nan }t
t{ -inf -inf f+      -> -inf }t

TESTING F-

t{ +0 +0 f- -> +0 }t
t{ +0 -0 f- -> +0 }t
t{ -0 +0 f- -> -0 }t
t{ -0 -0 f- -> +0 }t

t{ +nan  2e  f- -> +nan }t
t{ -nan  2e  f- -> -nan }t
t{  3e  +nan f- -> +nan }t
t{  3e  -nan f- -> -nan }t

t{ +nan +nan f-      -> +nan }t
t{ -nan +nan f- fabs -> +nan }t
t{ +nan -nan f- fabs -> +nan }t
t{ -nan -nan f-      -> -nan }t

t{  2e  +inf f- -> -inf }t
t{ +inf  7e  f- -> +inf }t
t{  2e  -inf f- -> +inf }t
t{ -inf  7e  f- -> -inf }t

t{ +nan +inf f- -> +nan }t
t{ +inf +nan f- -> +nan }t
t{ +nan -inf f- -> +nan }t
t{ -inf +nan f- -> +nan }t
t{ -nan +inf f- -> -nan }t
t{ +inf -nan f- -> -nan }t
t{ -nan -inf f- -> -nan }t
t{ -inf -nan f- -> -nan }t

t{ +inf +inf f- fabs -> +nan }t
t{ +inf -inf f-      -> +inf }t
t{ -inf +inf f-      -> -inf }t
t{ -inf -inf f- fabs -> +nan }t

TESTING F*

t{ +0 +0 f* -> +0 }t
t{ +0 -0 f* -> -0 }t
t{ -0 +0 f* -> -0 }t
t{ -0 -0 f* -> +0 }t

t{ +0  2e f* -> +0 }t
t{ -0  2e f* -> -0 }t
t{ +0 -2e f* -> -0 }t
t{ -0 -2e f* -> +0 }t
t{  2e +0 f* -> +0 }t
t{  2e -0 f* -> -0 }t
t{ -2e +0 f* -> -0 }t
t{ -2e -0 f* -> +0 }t

t{ +nan  2e  f* -> +nan }t
t{ -nan  2e  f* -> -nan }t
t{  3e  +nan f* -> +nan }t
t{  3e  -nan f* -> -nan }t

t{ +nan +nan f*      -> +nan }t
t{ -nan +nan f* fabs -> +nan }t
t{ +nan -nan f* fabs -> +nan }t
t{ -nan -nan f*      -> -nan }t

t{  2e  +inf f* -> +inf }t
t{ +inf  7e  f* -> +inf }t
t{  2e  -inf f* -> -inf }t
t{ -inf  7e  f* -> -inf }t

t{ +nan +inf f* -> +nan }t
t{ +inf +nan f* -> +nan }t
t{ +nan -inf f* -> +nan }t
t{ -inf +nan f* -> +nan }t
t{ -nan +inf f* -> -nan }t
t{ +inf -nan f* -> -nan }t
t{ -nan -inf f* -> -nan }t
t{ -inf -nan f* -> -nan }t

t{ +inf +inf f* -> +inf }t
t{ +inf -inf f* -> -inf }t
t{ -inf +inf f* -> -inf }t

TESTING F/

t{ +0 +0 f/ fabs -> +nan }t
t{ +0 -0 f/ fabs -> +nan }t
t{ -0 +0 f/ fabs -> +nan }t
t{ -0 -0 f/ fabs -> +nan }t

t{ +0  2e f/ -> +0 }t
t{ -0  2e f/ -> -0 }t
t{ +0 -2e f/ -> -0 }t
t{ -0 -2e f/ -> +0 }t
t{  2e +0 f/ -> +inf }t
t{  2e -0 f/ -> -inf }t
t{ -2e +0 f/ -> -inf }t
t{ -2e -0 f/ -> +inf }t

t{ +nan  2e  f/ -> +nan }t
t{ -nan  2e  f/ -> -nan }t
t{  3e  +nan f/ -> +nan }t
t{  3e  -nan f/ -> -nan }t

t{ +nan +nan f/      -> +nan }t
t{ -nan +nan f/ fabs -> +nan }t
t{ +nan -nan f/ fabs -> +nan }t
t{ -nan -nan f/      -> -nan }t

t{  2e  +inf f/ -> +0 }t
t{ +inf  7e  f/ -> +inf }t
t{  2e  -inf f/ -> -0 }t
t{ -inf  7e  f/ -> -inf }t

t{ +nan +inf f/ -> +nan }t
t{ +inf +nan f/ -> +nan }t
t{ +nan -inf f/ -> +nan }t
t{ -inf +nan f/ -> +nan }t
t{ -nan +inf f/ -> -nan }t
t{ +inf -nan f/ -> -nan }t
t{ -nan -inf f/ -> -nan }t
t{ -inf -nan f/ -> -nan }t

t{ +inf +inf f/ fabs -> +nan }t
t{ +inf -inf f/ fabs -> +nan }t
t{ -inf +inf f/ fabs -> +nan }t

TESTING FSQRT

t{ +0   fsqrt -> +0 }t
t{ -0   fsqrt -> -0 }t
t{ +inf fsqrt -> +inf }t
t{ -inf fsqrt -> -1e fsqrt }t
t{ +nan fsqrt -> +nan }t
t{ -nan fsqrt -> -nan }t
t{ -1e  fsqrt fnan? -> true }t

?." NOT TESTING F*+

?.errors GFORTH-HOST [IF] ?.cr [THEN]
